/**
 * the HTTP chunk-API
 * 
 * 
 */

#include <sys/types.h>
#include <sys/mman.h>
#include <sys/stat.h>

#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>

#include <stdio.h>
#include <errno.h>
#include <string.h>

#include "server.h"
#include "chunk.h"
#include "log.h"

int http_chunk_append_len(server *srv, connection *con, size_t len) {
	char hexbuf[65];
	
	ultohex(hexbuf, len);
	buffer_copy_string(srv->tmp_chunk_len, hexbuf);
#if 0
	/* just for debug purposes */
	buffer_append_string(srv->tmp_chunk_len, "; ");
	ultostr(hexbuf, len);
	buffer_append_string(srv->tmp_chunk_len, hexbuf);
#endif
	buffer_append_string(srv->tmp_chunk_len, "\r\n");
	chunk_append_buffer(con->write_chunk, srv->tmp_chunk_len);
	
	return 0;
}


int http_chunk_append_file(server *srv, connection *con, buffer *fn, off_t offset, off_t len) {
	chunkset *c;
	
	if (!con) return -1;
	
	c = con->write_chunk;
	
	if (con->response.transfer_encoding & HTTP_TRANSFER_ENCODING_CHUNKED) {
		http_chunk_append_len(srv, con, len);
	}
	
	chunk_append_file(c, fn, offset, len);
	
	if (con->response.transfer_encoding & HTTP_TRANSFER_ENCODING_CHUNKED && len > 0) {
		chunk_append_mem(c, "\r\n", 2 + 1);
	}
	
	return 0;
}

int http_chunk_append_buffer(server *srv, connection *con, buffer *mem) {
	chunkset *c;
	
	if (!con) return -1;
	
	c = con->write_chunk;
	
	if (con->response.transfer_encoding & HTTP_TRANSFER_ENCODING_CHUNKED) {
		http_chunk_append_len(srv, con, mem->used - 1);
	}
	
	chunk_append_buffer(c, mem);
	
	if (con->response.transfer_encoding & HTTP_TRANSFER_ENCODING_CHUNKED && mem->used > 0) {
		chunk_append_mem(c, "\r\n", 2 + 1);
	}
	
	return 0;
}

int http_chunk_append_mem(server *srv, connection *con, const char * mem, size_t len) {
	chunkset *c;
	
	if (!con) return -1;
	
	c = con->write_chunk;
	
	if (len == 0) {
		if (con->response.transfer_encoding & HTTP_TRANSFER_ENCODING_CHUNKED) {
			http_chunk_append_len(srv, con, 0);
			chunk_append_mem(c, "\r\n", 2 + 1);
		}
		return 0;
	}
	
	if (con->response.transfer_encoding & HTTP_TRANSFER_ENCODING_CHUNKED) {
		http_chunk_append_len(srv, con, len - 1);
	}
	
	chunk_append_mem(c, mem, len);
	
	if (con->response.transfer_encoding & HTTP_TRANSFER_ENCODING_CHUNKED) {
		chunk_append_mem(c, "\r\n", 2 + 1);
	}
	
	return 0;
}


size_t http_chunkset_length(server *srv, connection *con) {
	if (!con) {
		log_error_write(srv, __FILE__, __LINE__, "s", "connection is NULL!!");
		
		return 0;
	}
	
	return chunkset_length(con->write_chunk);
}
